# Introduction to R and RStudio
#===============================


# The Command Line
#------------------

2 + 2

(1 + 3 + 2 + 12 + 8)/5

( (2 - 1)^2 + (1 - 3)^2 )^(1/2)

2^3; 1 + 2 + 3

(1 + 3 + 2 + 12 + 
    8)/5

2^^3; 2^3

x <- 2
y <- x^2 - 2*x + 1
y

myData <- 5
Data_ <- 5
my.Data <- 5
my_Data <- data. <- 5

pi
pi <- 1000
pi
rm(pi)
pi

x <- pi
sin(x)
sqrt(x)

log(x)
log(x, 10)
log10(x)

x <- c(74, 122, 235, 111, 292)
mean(x)

x + x
sqrt(x)
x - mean(x)
sum(x - mean(x))

x1 <- c(74, 122, NA, 235, 111, 292)
mean(x1)
mean(x1, na.rm=TRUE)
mean(x1, trim=0.5, na.rm=TRUE)
mean(x1, 0.5, T)

x1 <- c(74, 122, NA, 235, 111, 292)
y <- c(FALSE, FALSE, TRUE, TRUE)
summary(x1)
summary(y)

?mean
help("mean")
apropos("mean")
example("mean")
help.search("mean")
help(package = "MASS")
vignette(package = "grid")


# The Workspace
#---------------

ls()
summary(x)
str(x)
rm(x)
ls()
rm(list=ls())


# External Packages
#-------------------

Sys.Date()
nrow(available.packages())

# Example: qcc package
# install the package on the system
install.packages("qcc")
# attach (load) the package to the workspace
library("qcc")
# alternatively, we can use
require("qcc")
# detach (unload) the package from the workspace
detach("package:qcc", unload=TRUE)
# remove the package from the system
# (usually not needed):
remove.packages("qcc") 


# Data Sets
#-----------

# data set rivers (package: datasets) 
head(rivers, n = 50)

# not shown in Workspace pane
rivers
rivers[1:5]
# editing produces a copy in workspace
rivers[1] <- NA
# remove
rm(rivers)
# load a copy into workspace
data(rivers)

data(package = "UsingR")
data(alltime.movies, package = "UsingR")
head(alltime.movies)


# Data Classes and Types
#------------------------

c(typeof(pi), mode(pi))
c(typeof("today"), mode("today"))
c(typeof(rivers), mode(rivers))

class(rivers)
summary(rivers)

class(mean)

class(alltime.movies)
summary(alltime.movies)

(myFactor1 <- factor(rep(1:5, 2), labels = letters[1:5]))
class(myFactor1)
(myFactor2 <- factor(rep(1:5, 2), labels = letters[1]))


# Vectors
#---------

x1 <- scan()
x1
str(x1)

x1 <- c(10, 20, 30, 40, 50); x1
x1 <- seq(from = 10, to = 50, by = 10); x1
x2 <- 1:10; x2
x3 <- c(rep("pinetree", 3), rep("oaktree", 2)); x3
x4 <- c(seq(from = 0, to = 1, by = 0.2), 5:9); x4
x5 <- seq_along(x4); x5

rep(c(1,2,3), times = c(3,2,1))

logicalVector <- 1:6 > 3
logicalVector
sum(logicalVector)

length(x1)

names(x1) <- paste("week", 1:5, sep=""); x1
names(x1)

precip
length(precip)
names(precip)

x1[3]
x1[3] <- 50; x1
x1[c(1,3)]
x1[c(-2)]
x1["week3"]
# extend a vector
x1 <- c(x1, 60); x1
x1[10] <- 100; x1
# reduce a vector
x1 <- x1[-c(6:10)]; x1
# use a logical expression
x1[x1 > 20]

precip[c("Seattle Tacoma", "New York")]

# find all cities with an average precipitation of 
# more than 50 inches
precip[precip > 50]

cd <- c(79,74,161,127,133,210,99,143,249,249,368,302)
names(cd) <- month.abb; cd
m31 <- c(1,3,5,7,8,10,12)
cd31 <- cd[m31]; cd31
cd30 <- cd[-m31]; cd30

set.seed(1234)
x6 <- sample(1:1000, 10, replace = TRUE); x6
sort(x6)
sort(x6, decreasing = TRUE)
ii <- order(x6); ii
x6[ii]

mean(x6)
x6 + 100
sqrt(x6)


# Matrices
#----------

myMatrix1 <- matrix(c(1:15), nrow=3, ncol=5); myMatrix1
myMatrix2 <- matrix(c(1:15), nrow=3, byrow=T); myMatrix2

myMatrix1[3, 2]
myMatrix1[1, ]
myMatrix1[ , 2]
myMatrix1[2:3, ]
myMatrix1[, 5] <- myMatrix2[, 5]; myMatrix1

colnames(myMatrix2) <- paste("var", 1:5, sep="")
rownames(myMatrix2) <- paste("case", 1:3, sep="")
myMatrix2

myMatrix2["case3", ]

# matrices have a 'dim' attribute 
dim(myMatrix2)
dim(myMatrix2) <- c(5,3)
myMatrix2
dim(myMatrix2) <- c(3,5)
myMatrix2

# row/column sums and means
rowSums(myMatrix2)
rowMeans(myMatrix2)
colMeans(myMatrix2)
# transposition
t(myMatrix2)
# elementwise multiplication
myMatrix2 * myMatrix2
# true matrix mulitiplication
myMatrix2 %*% t(myMatrix2)

# 3x3 Hilbert matrix
h3 <- matrix(c(  1,1/2,1/3,
               1/2,1/3,1/4,
               1/3,1/4,1/5), ncol=3)
h3
# inverse of h3
solve(h3)


# Lists
#-------

(myList <- list(mat = myMatrix2, vec1 = x1, vec2 = 1:10))
myList$vec1
myList[3]
myList[[3]]
myList[[3]][1]
myList[c(1, 2)]
myList$mat[ , 2]
myList$mat[ , 2, drop = FALSE]
str(myList)


# Data Frames
#-------------

myData <- data.frame(
  type = c("A", "A", "B", "C", "C", "C"), 
  weight = c(10.1,20.3,15.2,13.4,23.2,8.1))
myData
str(myData)

myData$type   # results in a vector
myData[[1]]   # results in a vector
myData[2]     # results in a data frame
myData[3, ]
myData[myData$weight < 15, ]

myEditedData <- myData
colnames(myEditedData) 
colnames(myEditedData)[2] <- "itemWeight"
rownames(myEditedData)
rownames(myEditedData) <- paste("case", 
                                rownames(myEditedData), 
                                sep = "_")
myEditedData


myData[order(myData$weight), ]

subset(myData, weight > 15)

aggregate(weight ~ type, data = myData, sum)

myData
myData[3, 2] <- 22.2
myData

set.seed(1)
myData$randomorder <- sample(1:6)
myData

myData$randomorder <- NULL
myData

myData$proportion <- myData$weight/sum(myData$weight)
myData


# Tibbles
#---------

#library("tibble")
as.tibble(myData)
is.tibble(as.tibble(myData))
is.data.frame(as.tibble(myData))

is.tibble(ggplot2::mpg)
is.data.frame(ggplot2::mpg)
ggplot2::mpg


# Data Import
#-------------

download.file(
  url = "http://emilio.lcano.com/qcrbook/lab.csv",
  destfile = "lab.csv")
lab <- read.csv("lab.csv")

# handling a long address
url_1 <- "https://raw.githubusercontent.com/hadley/"
url_2 <- "r4ds/master/data/heights.csv"
url_heights <- paste(url_1, url_2, sep="")
download.file(url=url_heights, destfile="heights.csv")
heights <- read.csv("heights.csv")

require(Quandl)
at_0014 <- Quandl("WWDI/AUT_SP_POP_0014_TO_ZS")
at_1564 <- Quandl("WWDI/AUT_SP_POP_1564_TO_ZS")
at_65up <- Quandl("WWDI/AUT_SP_POP_65UP_TO_ZS")
at_all <- Reduce(function(x, y) merge(x, y, by="Date"),
                 list(at_0014, at_1564, at_65up))
names(at_all) <- c("Date", "[0,14]","[15,64]","[65,)")
heights <- t(at_all[,-1])
colnames(heights) <- format(at_all[,"Date"], format="%Y")
barplot(heights, main="AUT: Proportion of 0-14, 15-64, 65-",
        col=rainbow(3))

require(XML)
require(xml2)
url_base = "https://en.wikipedia.org/wiki/"
url_add1 <- "List_of_highest-grossing_films_in_"
url_add2 <- "Canada_and_the_United_States"
url_all <- paste(url_base, url_add1, url_add2, sep="")
u <- read_html(url_all)
doc <- htmlParse(u)
tableNodes <- getNodeSet(doc, "//table")

# first table
tb1 <- readHTMLTable(tableNodes[[1]])
View(tb1)

# second table
tb2 <- readHTMLTable(tableNodes[[2]])
View(tb2)


# Graphics
#----------

with(iris, plot(Sepal.Length, Sepal.Width,
                pch=as.numeric(Species), 
                col=as.numeric(Species), cex=1.2))
legend(6.1, 4.4, c("setosa","versicolor","virginica"),
       cex=1, pch=1:3, col=1:3)

fm <- Sepal.Width ~ Sepal.Length
plot(fm, iris, pch=as.numeric(Species), 
     col=as.numeric(Species))
out <- mapply(function(i, x) abline(lm(fm, data=x), 
              lty=i, lwd=2, col=i), i=1:3, 
              x=split(iris, iris$Species))
legend(6.4, 4.4, levels(iris$Species), cex=1,
       lty=1:3, col=1:3)


require(lattice)
xyplot(Sepal.Width ~ Sepal.Length | Species, data=iris)

xyplot(Sepal.Width ~ Sepal.Length | Species, data=iris,
       panel = function(x, y) {
          panel.xyplot(x, y)
          panel.lmline(x, y)
       })

xyplot(Sepal.Width ~ Sepal.Length | Species, 
       data=iris, pch=19, cex=1.1, lwd=2, 
       layout=c(3,1), type=c("p", "r"))


require(ggplot2)

ggplot2::mpg

ggplot(data = mpg) + 
  geom_point(mapping = aes(x = displ, y = hwy, color = class),
             size = 2.3)
  
ggplot(data = iris, 
  mapping = aes(x = Sepal.Length, y = Sepal.Width)) +
geom_point(mapping = aes(color = Species), size = 2.7) +
geom_smooth(mapping = aes(group = Species, color = Species))
